@@ -22,6 +22,7 @@
 
 #include <config.h>
 #include "window-props.h"
+#include "errors.h"
 #include "xprops.h"
 #include "frame.h"
 #include "group.h"
@@ -248,9 +249,13 @@ set_title_text (MetaWindow  *window,
 
   /* Bug 330671 -- Don't forget to clear _NET_WM_VISIBLE_(ICON_)NAME */
   if (!modified && previous_was_modified)
-    XDeleteProperty (window->display->xdisplay,
-                     window->xwindow,
-                     atom);
+    {
+      meta_error_trap_push (window->display);
+      XDeleteProperty (window->display->xdisplay,
+                       window->xwindow,
+                       atom);
+      meta_error_trap_pop (window->display, FALSE);
+    }
 
   return modified;
 }
@@ -687,12 +692,31 @@ static void
 reload_net_startup_id (MetaWindow    *window,
                        MetaPropValue *value)
 {
+  guint32 timestamp = window->net_wm_user_time;
+  MetaWorkspace *workspace = NULL;
+  
   g_free (window->startup_id);
   
   if (value->type != META_PROP_VALUE_INVALID)
     window->startup_id = g_strdup (value->v.str);
   else
     window->startup_id = NULL;
+    
+  /* Update timestamp and workspace on a running window */
+  if (!window->constructing)
+  {
+    window->initial_timestamp_set = 0;  
+    window->initial_workspace_set = 0;
+    
+    meta_screen_apply_startup_properties (window->screen, window);
+  
+    if (window->initial_timestamp_set)
+      timestamp = window->initial_timestamp;
+    if (window->initial_workspace_set)
+      workspace = meta_screen_get_workspace_by_index (window->screen, window->initial_workspace);
+    
+    meta_window_activate_with_workspace (window, timestamp, workspace);
+  }
   
   meta_verbose ("New _NET_STARTUP_ID \"%s\" for %s\n",
                 window->startup_id ? window->startup_id : "unset",
@@ -1155,10 +1179,10 @@ reload_transient_for (MetaWindow    *window,
       meta_display_lookup_x_window (window->display, 
                                     window->xtransient_for) == NULL)
     {
-      window->xtransient_for = None;
       meta_warning (_("Invalid WM_TRANSIENT_FOR window 0x%lx specified "
                       "for %s.\n"),
                     window->xtransient_for, window->desc);
+      window->xtransient_for = None;
     }
 
   window->transient_parent_is_root_window =
